#!/usr/bin/env python
# Turbo Birdie Interceptor
# A Game for PyWeek (made in three days...)
# Andrew Lindberg
# 

import pygame
import os
import sys
import time
import math
import random
from pygame.locals import *

class Spritesheet:
  def __init__(self):
    self.sheet = pygame.image.load("sheet.png").convert_alpha()
    
    self.bat = pygame.Surface((20,13)).convert()
    self.bat.blit(self.sheet,(0,0),(102,15,20,13))
    self.bat.set_colorkey((255,0,255))
    self.bat.convert_alpha()
    
    self.feather = pygame.Surface((15,18)).convert()
    self.feather.blit(self.sheet,(0,0),(13,11,15,18))
    self.feather.set_colorkey((255,0,255))
    self.feather.convert_alpha()
    
    self.balloon = pygame.Surface((16,19)).convert()
    self.balloon.blit(self.sheet,(0,0),(31,9,16,19))
    self.balloon.set_colorkey((255,0,255))
    self.balloon.convert_alpha()
    
    self.bird = pygame.Surface((44,20)).convert()
    self.bird.blit(self.sheet,(0,0),(50,9,44,20))
    self.bird.set_colorkey((255,0,255))
    self.bird.convert_alpha()
  def get_bat(self):
    return self.bat
  def get_feather(self):
    return self.feather
  def get_bird(self):
    return self.bird
  def get_balloon(self):
    return self.balloon

class Player(pygame.sprite.Sprite):
  MAXSPEED = 30
  speed = 0
  images = []
  bankspd = 6
  def __init__(self, screenrect,image):
    pygame.sprite.Sprite.__init__(self)
    self.image = image
    self.rect = self.image.get_rect()
    self.scr_rect = screenrect
  def set_bank_speed(self,spd):
    self.bankspd = min(spd, self.MAXSPEED)
  def update(self):
    self.rect.move_ip((self.speed,0))
    self.rect.clamp_ip(self.scr_rect)
  def place_at(self,x,y):
    self.rect.center = (x,y)

class Baddie(pygame.sprite.Sprite):
  speed = 0
  def __init__(self,image):
    pygame.sprite.Sprite.__init__(self)
    self.image = image
    self.rect = self.image.get_rect()
  def update(self):
    self.rect.move_ip((0,self.speed))
    if self.rect.top > 500:
      self.kill()
  def set_speed(self,spd):
    self.speed = spd
  def place_at(self,x,y):
    self.rect.center = (x,y)

class Item(pygame.sprite.Sprite):
  speed = 0
  def __init__(self, image):
    pygame.sprite.Sprite.__init__(self)
    self.image = image
    self.rect = self.image.get_rect()
  def update(self):
    self.rect.move_ip((0,self.speed))
    if self.rect.top > 500:
      self.kill()
  def set_speed(self,spd):
    self.speed = spd
  def place_at(self,x,y):
    self.rect.center = (x,y)

def get_timer():
  # borrowed from the timeit module at python.org
  if sys.platform == "win32":
    # On Windows, the best timer is time.clock()
    default_timer = time.clock
  else:
    # On most other platforms the best timer is time.time()
    default_timer = time.time
    return default_timer


class Menu:
  def __init__(self,scr):
    self.screen = scr
    self.largefont = pygame.font.Font(None,48)
    self.font = pygame.font.Font(None,24)
    self.buffer = pygame.Surface(self.screen.get_size()).convert_alpha()
    self.selector = pygame.Surface((12,12))
    self.selector.fill((255,255,255))
    self.background = pygame.Surface(self.screen.get_size()).convert()
    self.background.fill((150,30,30))
    self.spritesheet = Spritesheet()
    
  def loop(self):
    running = True
    level_index = 0
    feathers = None
    time = None
    white = (255,255,255)
    red = (150,30,30)
    titletext = self.largefont.render("Turbo Feather Interceptor",1,white,red)
    select_pos = [(60,300),(60,324)]
    sel_index = 0
    helptext = self.font.render("Left/Right to move.",1,white,red)
    helptext2 = self.font.render("Collect Feathers:",1,white,red)
    helptext3 = self.font.render("Avoid the Rest:",1,white,red)
    nextlevel = self.font.render("Play Next Level",1,white,red)
    firstlevel = self.font.render("Play",1,white,red)
    quittext = self.font.render("Quit",1,white,red)
    while(running):
      # handle input
      for event in pygame.event.get():
        if event.type == QUIT:
          running = False
        elif event.type == KEYDOWN:
          if event.key == K_UP:
            sel_index = max(sel_index-1,0)
          elif event.key == K_DOWN:
            sel_index = min(len(select_pos)-1,sel_index+1)
          elif event.key == K_q:
            running = False
          elif event.key == K_p or event.key == K_RETURN:
            if sel_index == 1:
              running = False
            else:
              level = Level(self.screen,self.spritesheet)
              feathers,time = level.loop()
              level_index = max(level_index+1,1)
      # render
      self.buffer.blit(self.background,(0,0))
      self.buffer.blit(titletext,(40,40))
      # print level results or intro text
      if level_index == 0:
        self.buffer.blit(helptext,(60,100))
        self.buffer.blit(helptext2,(60,130))
        self.buffer.blit(helptext3,(60,160))
        self.buffer.blit(firstlevel,(75,300))
      else:
        leveltext1 = self.largefont.render("Level %d Complete" % level_index,
                                            1,white,red)
        leveltext2 = self.font.render("Collected %d Feathers" % 
                                      feathers,1,white,red)
        leveltext3 = self.font.render(" in %.2f seconds." % time,1,white,red)
        self.buffer.blit(leveltext1,(120,150))
        self.buffer.blit(leveltext2,(120,200))
        self.buffer.blit(leveltext3,(120,230))
        self.buffer.blit(nextlevel,(75,300))
      self.buffer.blit(quittext,(75,324))
      self.buffer.blit(self.selector,select_pos[sel_index])
      self.screen.blit(self.buffer,(0,0))
      pygame.display.flip()


class Level:
  def __init__(self,scr,spritesheet,tables=None):
    self.screen = scr
    self.MINSPEED = 4
    self.MAXSPEED = 20
    self.GOAL_DIST = 40000
    self.speed = 8
    self.distance = 0

    self.timer = get_timer()
    self.spr_sheet = spritesheet
    self.buffer = pygame.Surface(self.screen.get_size()).convert_alpha()
    self.allsprites = pygame.sprite.Group()
    self.feathers = pygame.sprite.Group()
    self.baddies = pygame.sprite.Group()
    # text stuff
    self.font = pygame.font.Font(None,24)
    # sky stuff
     
    self.skyoffset = 0
    self.skyimage = pygame.image.load('sky.png').convert() 
    self.sky = pygame.Surface((640,960)).convert()
    self.sky.blit(self.skyimage,(0,0))
    self.buffer.blit(self.sky,(0,0))
    
    #hud stuff
    self.meter = pygame.Surface((16,400)).convert_alpha()
    self.meter.fill((255,255,255,100))
    self.meter.fill((200,100,100,100),(2,400-1,12,1))
    self.meter.set_alpha(200)
    # player stuff
    self.player = Player(self.screen.get_rect(),self.spr_sheet.get_bird())
    self.player.place_at( self.screen.get_width()/2, self.screen.get_height()-48)
    
    self.allsprites.add(self.player)
    self.running = True
    
    # level vars
    self.SKY_INDEX = 0
    self.total_feathers = 0
    self.feathercount = 0
    self.collisions = 0
    # orig
    #self.fprob_table = [0.30, 0.30, 0.30, 0.25, 0.20, 0.12]
    #self.bprob_table = [0.10, 0.10, 0.20, 0.20, 0.25, 0.30]
    #self.b_spd_table = [0.80, 0.80, 1.00, 1.00, 1.20, 1.25]
    self.speed_table = [10,12,14,16,20,24]
    self.fprob_table = [0.30, 0.30, 0.30, 0.25, 0.25, 0.15]
    self.bprob_table = [0.00, 0.00, 0.00, 0.00, 0.10, 0.15]
    self.b_spd_table = [1.00, 1.00, 1.00, 1.00, 1.00, 1.00]
    # prior to speed table
    #self.fprob_table = [0.25, 0.25, 0.25, 0.25, 0.20, 0.20]
    #self.bprob_table = [0.00, 0.00, 0.00, 0.00, 0.05, 0.10]
    #self.b_spd_table = [0.80, 0.80, 1.00, 1.00, 1.20, 1.25]
    self.clock = pygame.time.Clock()
    
  def loop(self):
    start_time = self.timer()
    while(self.distance < self.GOAL_DIST and self.running):
      elapsed_time = self.timer()-start_time
      for event in pygame.event.get():
        if event.type == QUIT:
          self.running = False
        elif event.type == KEYDOWN:
          if event.key == K_q:
            self.running = False
          elif event.key == K_LEFT:
            if self.player.speed > 0:
              self.player.speed = 0
            else:
              self.player.speed -= self.player.bankspd
          elif event.key == K_RIGHT:
            if self.player.speed < 0:
              self.player.speed = 0
            else:
              self.player.speed += self.player.bankspd
        elif event.type == KEYUP:
          if event.key == K_LEFT:
            if self.player.speed == 0:
              self.player.speed += self.player.bankspd
            else:
              self.player.speed = 0
          elif event.key == K_RIGHT:
            if self.player.speed == 0:
              self.player.speed -= self.player.bankspd
            else:
              self.player.speed = 0
      self.speed = self.speed_table[self.SKY_INDEX]
      feather_text = "Feathers: %d" % self.total_feathers
      time_text = "%.2f Elapsed" % elapsed_time
      feather_txt_surf = self.font.render(feather_text,1,(255,255,255),
                                          (150,30,30))
      feather_txt_surf.convert()
      time_txt_surf = self.font.render(time_text,1,(255,255,255),(150,30,30))
      time_txt_surf.convert()
      self.player.set_bank_speed(self.speed*0.75)
      # feathers
      if random.random() < self.fprob_table[self.SKY_INDEX]:
        i = Item(self.spr_sheet.get_feather())
        i.place_at(20+random.random()*600, 30)
        i.set_speed(self.speed*0.8)
        self.allsprites.add(i)
        self.feathers.add(i)
      for each in pygame.sprite.spritecollide(self.player,self.feathers,1):
        self.feathercount += 1
        self.total_feathers += 1
      # baddies
      if random.random() < self.bprob_table[self.SKY_INDEX]:
        b = Baddie(self.spr_sheet.get_bat())
        b.place_at(20+random.random()*600, 30)
        b.set_speed(self.speed*self.b_spd_table[self.SKY_INDEX])
        self.allsprites.add(b)
        self.baddies.add(b)
      for each in pygame.sprite.spritecollide(self.player,self.baddies,1):
        self.collisions += 1
        self.total_feathers = max( 0, self.total_feathers-2)

      # handle collsions and feathers
      if self.collisions >= 3:
        self.collisions = 0
        self.feathercount = 0
        self.SKY_INDEX = max(0, self.SKY_INDEX-1)
      if self.feathercount >= 10:
        self.feathercount = 0
        self.SKY_INDEX = min( self.SKY_INDEX+1, 5)
      
      # update the screen
      self.allsprites.update()
      self.skyoffset = ((self.skyoffset - self.speed) % 480)
      self.buffer.blit(self.sky,(0,0),(0,self.skyoffset,self.screen.get_width(),
        self.screen.get_height()))
      partial = math.ceil((1.0*self.distance)/(1.0*self.GOAL_DIST)*400.0)
      self.meter.fill((200,100,100,100),rect=pygame.Rect(2,400-partial,12,5))
      self.buffer.blit(self.meter,(615,40))
      self.allsprites.draw(self.buffer)
      self.buffer.blit(feather_txt_surf,(20,450))
      self.buffer.blit(time_txt_surf,(220,450))
      self.screen.blit(self.buffer,(0,0))
      self.distance += self.speed
      pygame.display.flip()
    return (self.total_feathers,elapsed_time)

def main():

  pygame.display.init()
  pygame.font.init()
  SCREENRECT = pygame.Rect(0,0,640,480)
  screen = pygame.display.set_mode(SCREENRECT.size)

  menu = Menu(screen)
  menu.loop()

if __name__ == '__main__':
  main()
